/* This file contains various defines for the FAS async driver.
   If you change anything here you have to recompile the driver module.
*/

#if !defined (M_I286)
#ident	"@(#)fas.h	2.12"
#endif

/* Uncomment the following line if you need (asy|sio)putchar and
   (asy|sio)getchar. This is only required if you link the kernel
   without the original asy or sio driver and these functions aren't
   provided by any other kernel module.
*/
/* #define NEED_PUT_GETCHAR	/* */

/* Uncomment the following line if you have VP/ix support in the
   kernel.
*/
#define HAVE_VPIX	/* */

/* Uncomment the following line if there is a tunable `ttyhog' variable
   in the kernel.
*/
#define TUNABLE_TTYHOG	/* */

/* Uncomment the following line if you want FAS to make sure that the
   thresholds in tthiwat [] and ttlowat [] are high enough to prevent
   the CLIST buffer functions to become a bottleneck at high baud rates.
   tthiwat [] and ttlowat [] are global resources and are used by other
   tty drivers as well, so in case that fixing these values causes
   problems with these drivers this feature is configurable.
*/
#define FIX_TTHILOWAT	/* */

/* Uncomment the following line if you need init8250. DosMerge needs
   this function, but only if you link the kernel without the original
   asy or sio driver.
*/
/* #define NEED_INIT8250	/* */

/* Initial line control register. This value will only be meaningful if
   NEED_PUT_GETCHAR is defined.
*/
#define	INITIAL_LINE_CONTROL	LC_WORDLEN_8

/* Initial baud rate. This value will only be meaningful if
   NEED_PUT_GETCHAR is defined.
*/
#define INITIAL_BAUD_RATE	B9600

/* Initial modem control register. This should probably not have to
   be touched. It is here because some terminals used as the console
   require one or more of the modem signals set. It is only meaningful
   if NEED_PUT_GETCHAR is defined.
*/
#define INITIAL_MDM_CONTROL	0

/****************************************************/
/* Nothing past this line should have to be changed */
/****************************************************/

/* The following section is, if necessary, automatically updated with
   OS dependent informations. Don't change it by hand!
*/
/* @@OS_DEP_BEGIN@@ */
/* @@OS_DEP_END@@ */

#if defined (VPIX)
#undef VPIX
#endif

#if defined (HAVE_VPIX)
#define VPIX
#endif

#if defined (SCO) && !defined (_NO_PROTOTYPE)
#define _NO_PROTOTYPE 1
#endif

#include <sys/param.h>
#include <sys/types.h>
#include <sys/signal.h>
#include <sys/buf.h>
#include <sys/dir.h>
#if defined (XENIX)
#include <sys/page.h>
#include <sys/seg.h>
#endif
#include <sys/user.h>
#include <sys/errno.h>
#include <sys/tty.h>
#include <sys/conf.h>
#include <sys/sysinfo.h>
#include <sys/file.h>
#if !defined (XENIX)
#include <sys/termio.h>
#endif
#if defined (SCO) || defined (XENIX)
#include <sys/select.h>
#endif
#include <sys/ioctl.h>
#if !defined (M_I286)
#if !defined (XENIX)
#include <sys/sysmacros.h>
#include <sys/tss.h>
#include <sys/immu.h>
#include <sys/region.h>
#endif
#include <sys/proc.h>
#if defined (HAVE_VPIX)
#include <sys/v86.h>
#endif
#endif

#if defined (XENIX)
typedef unsigned char	unchar;
typedef unsigned long	ulong;

/* Union for use by all device handler ioctl routines. */
union ioctl_arg {
	struct termio	*stparg;	/* ptr to termio struct */
	char		*cparg;		/* ptr to character */
	char		carg;		/* character */
	int		*iparg;		/* ptr to integer */
	int		iarg;		/* integer */
	long            *lparg;         /* ptr to long */
	long            larg;           /* long */
};
#endif

#if defined (__GNUC__)
/* in expressions GNU C is able to use operand sizes < native integer size */
typedef unchar	n_unchar;
#else
/* in expressions AT&T C extends smaller types to the native integer size */
typedef uint	n_unchar;
#endif

/* allow function prototypes in ANSI C */
#if defined (__STDC__)
#define P(x) x
#else
#define P(x) ()
#endif

#if defined (TRUE)
#undef TRUE
#endif
#define	TRUE	(1)

#if defined (FALSE)
#undef FALSE
#endif
#define FALSE	(0)

typedef int	bool;

/* Miscellaneous Constants */

#define HANGUP_DELAY		500	/* in milli-seconds */
#define HANGUP_TIME		1000	/* in milli-seconds */
#define BREAK_TIME		250	/* in milli-seconds */
#define MAX_EVENT_TIME		10	/* in milli-seconds */
#define MAX_RXFER_DELAY		20	/* in milli-seconds */
#define RECV_BUFF_FILL_TIME	2000	/* in milli-seconds */
#define XMIT_BUFF_DRAIN_TIME	2000	/* in milli-seconds */
#if defined (M_I286)
#define	RECV_BUFF_MAX_SIZE	1000	/* receiver ring buffer max size */
#define XMIT_BUFF_MAX_SIZE	500	/* transmitter ring buffer max size */
#else
#define	RECV_BUFF_MIN_SIZE	1000	/* receiver ring buffer min size */
#define	RECV_BUFF_MAX_SIZE	10000	/* receiver ring buffer max size */
#define XMIT_BUFF_MAX_SIZE	10000	/* transmitter ring buffer max size */
#endif
#define RECV_PC_SW_LOW_WATER	50	/* recv sw fc low water level in % */
#define RECV_PC_SW_HIGH_WATER	80	/* recv sw fc high water level in % */
#define RECV_PC_HW_LOW_WATER	65	/* recv hw fc low water level in % */
#define RECV_PC_HW_HIGH_WATER	95	/* recv hw fc high water level in % */
#define SOFT_INIT		0	/* init registers if cflag changed */
#define HARD_INIT		1	/* init registers w/o checking cflag */
#define TEST_CHAR_LOOPS		100	/* no. of character test loops */
#define TEST_CTL_LOOPS		1000	/* no. of control line test loops */
#define TEST_DELAY_LOOPS	60000	/* no. of delay loops */
#define TEST_INTR_LOOPS		1000	/* no. of interrupt test loops */
#if defined (SCO) || defined (XENIX)
#define SPLWRK			spl5	/* SPL for character processing */
#define SPLINT			spl7	/* SPL to disable FAS interrupts */
#else
#define SPLWRK			spl6	/* SPL for character processing */
#define SPLINT			spltty	/* SPL to disable FAS interrupts */
#endif
#define MAX_UNITS		16	/* max. number of physical units */
#define MAX_BAUD_TABLES		256	/* max. number of baud rate tables */
#define NUM_UART_REGS		7	/* number of UART registers */

/* calculate EVENT_TIME from MAX_EVENT_TIME with regard to the
   clock tick granularity
*/

#define EVENT_TIME	((((MAX_EVENT_TIME) * (HZ) / 1000)\
				* 1000 + (HZ) - 1) / (HZ))
#if EVENT_TIME == 0
#undef EVENT_TIME
#define EVENT_TIME	((1000 + (HZ) - 1) / (HZ))
#endif

#if MAX_RXFER_DELAY < EVENT_TIME
#undef MAX_RXFER_DELAY
#define MAX_RXFER_DELAY	EVENT_TIME
#endif

#define MAX_INPUT_FIFO_SIZE	INPUT_NS_FIFO_SIZE
#define MAX_OUTPUT_FIFO_SIZE	OUTPUT_NS_FIFO_SIZE


/* Here are the modem control flags for the fas_modem array in space.c.
   They are arranged in three 8-bit masks which are combined to a 32-bit
   word. Each of these 32-bit words represents one entry in the fas_modem
   array.

   The lowest byte is used as a mask to manipulate the modem control
   register for modem disable. Use the MC_* macros to build the mask.

   The second lowest byte is used as a mask to manipulate the modem control
   register for modem enable during dialout. Use the MC_* macros to build
   the mask and shift them 8 bits to the left.

   The second highest byte is used as a mask to manipulate the modem control
   register for modem enable during dialin. Use the MC_* macros to build
   the mask and shift them 16 bits to the left.

   The highest byte is used to mask signals from the modem status
   register that will be used as the carrier detect signal. Use the MS_*
   macros to build the mask and shift them 24 bits to the left. If you use
   more than one signal, carrier is considered on only when all signals
   are on.

   Here are some useful macros for the space.c file. You may create your
   own macros if you have some special requirements not met by the
   predefined ones.
*/

/* modem disable (choose one) */
#define DI_RTS			((ulong) MC_SET_RTS)
#define DI_DTR			((ulong) MC_SET_DTR)
#define DI_RTS_AND_DTR		((ulong) (MC_SET_RTS | MC_SET_DTR))

/* modem enable for dialout (choose one) */
#define EO_RTS			((ulong) MC_SET_RTS << 8)
#define EO_DTR			((ulong) MC_SET_DTR << 8)
#define EO_RTS_AND_DTR		((ulong) (MC_SET_RTS | MC_SET_DTR) << 8)

/* modem enable for dialin (choose one) */
#define EI_RTS			((ulong) MC_SET_RTS << 16)
#define EI_DTR			((ulong) MC_SET_DTR << 16)
#define EI_RTS_AND_DTR		((ulong) (MC_SET_RTS | MC_SET_DTR) << 16)

/* carrier detect signal (choose one) */
#define CA_DCD			((ulong) MS_DCD_PRESENT << 24)
#define CA_CTS			((ulong) MS_CTS_PRESENT << 24)
#define CA_DSR			((ulong) MS_DSR_PRESENT << 24)


/* Here are the hardware handshake flags for the fas_flow array in space.c.
   They are arranged in three 8-bit masks which are combined to a 32-bit
   word. Each of these 32-bit words represents one entry in the fas_flow
   array.

   The lowest byte is used as a mask to manipulate the modem control
   register for input flow control. Use the MC_* macros to build the mask.

   The second lowest byte is used to mask signals from the modem status
   register that will be used for output flow control. Use the MS_* macros
   to build the mask and shift them 8 bits to the left. If you use more
   than one signal, output is allowed only when all signals are on.

   The second highest byte is used to mask signals from the modem status
   register that will be used to enable the output flow control selected
   by the second lowest byte. Use the MS_* macros to build the mask and
   shift them 16 bits to the left. If you use more than one signal, output
   flow control is enabled only when all signals are on.

   The highest byte is used as a mask to manipulate the modem control
   register for output half duplex flow control. Use the MC_* macros to
   build the mask and shift them 24 bits to the left.

   Here are some useful macros for the space.c file. You may create your
   own macros if you have some special requirements not met by the
   predefined ones.
*/

/* input flow control (choose one) */
#define HI_RTS			((ulong) MC_SET_RTS)
#define HI_DTR			((ulong) MC_SET_DTR)
#define HI_RTS_AND_DTR		((ulong) (MC_SET_RTS | MC_SET_DTR))

/* output flow control (choose one) */
#define HO_CTS			((ulong) MS_CTS_PRESENT << 8)
#define HO_DSR			((ulong) MS_DSR_PRESENT << 8)
#define HO_CTS_AND_DSR		((ulong) (MS_CTS_PRESENT | MS_DSR_PRESENT) \
					<< 8)
#define HO_CTS_ON_DSR		(((ulong) MS_CTS_PRESENT << 8) \
				| ((ulong) MS_DSR_PRESENT << 16))
#define HO_CTS_ON_DSR_AND_DCD	(((ulong) MS_CTS_PRESENT << 8) \
				| ((ulong) (MS_DSR_PRESENT | MS_DCD_PRESENT) \
					<< 16)) 

/* output hdx flow control (choose one) */
#define HX_RTS			((ulong) MC_SET_RTS << 24)
#define HX_DTR			((ulong) MC_SET_DTR << 24)
#define HX_RTS_AND_DTR		((ulong) (MC_SET_RTS | MC_SET_DTR) << 24)


/* define the local open flags */

#define OS_DEVICE_CLOSED	0x0000
#define OS_OPEN_FOR_DIALOUT	0x0001
#define OS_OPEN_FOR_DIALIN	0x0002
#define OS_WAIT_OPEN		0x0004
#define OS_NO_DIALOUT		0x0008
#define OS_FAKE_CARRIER_ON	0x0010
#define OS_CLOCAL		0x0020
#define OS_HWO_HANDSHAKE	0x0040
#define OS_HWI_HANDSHAKE	0x0080
#define OS_HDX_HANDSHAKE	0x0100
#define OS_EXCLUSIVE_OPEN_1	0x0200
#define OS_EXCLUSIVE_OPEN_2	0x0400	/* SYSV 3.2 Xenix compatibility */

#define OS_OPEN_STATES		(OS_OPEN_FOR_DIALOUT | OS_OPEN_FOR_DIALIN)
#define OS_TEST_MASK		(OS_OPEN_FOR_DIALOUT | OS_NO_DIALOUT \
				| OS_FAKE_CARRIER_ON | OS_CLOCAL \
				| OS_HWO_HANDSHAKE | OS_HWI_HANDSHAKE \
				| OS_HDX_HANDSHAKE | OS_EXCLUSIVE_OPEN_1 \
				| OS_EXCLUSIVE_OPEN_2)
#define OS_SU_TEST_MASK		(OS_OPEN_FOR_DIALOUT | OS_NO_DIALOUT \
				| OS_FAKE_CARRIER_ON | OS_CLOCAL \
				| OS_HWO_HANDSHAKE | OS_HWI_HANDSHAKE \
				| OS_HDX_HANDSHAKE | OS_EXCLUSIVE_OPEN_1)

/* define the device status flags */

#define DF_CTL_FIRST		0x0001	/* write ctl port at first access */
#define DF_CTL_EVERY		0x0002	/* write ctl port at every access */
#define DF_XMIT_BUSY		0x0004	/* transmitter busy */
#define DF_XMIT_BREAK		0x0008	/* transmitter sends break */
#define DF_XMIT_LOCKED		0x0010	/* transmitter locked against output */
#define DF_GUARD_TIMEOUT	0x0020	/* protect last char from corruption */
#define DF_MSI_NOISE		0x0040	/* modem status interrupts noise */
#define DF_MSI_ENABLED		0x0080	/* modem status interrupts enabled */
#define DF_RDI_ENABLED		0x0100	/* receiver data interrupts enabled */
#define DF_HUP_PROTECT		0x0200	/* protect device after hangup */
#define DF_NO_OVERRUN		0x0400	/* device is overrun protected */
#define DF_DEVICE_CONFIGURED	0x0800	/* device is configured */
#define DF_DEVICE_OPEN		0x1000	/* physical device is open */
#define DF_DEVICE_LOCKED	0x2000	/* physical device locked */
#define DF_DO_HANGUP		0x4000	/* delayed hangup request */
#define DF_DO_BREAK		0x8000	/* delayed break request */

/* define the flow control status flags */

#define FF_HWO_HANDSHAKE	0x0001	/* output hw handshake enabled */
#define FF_HWI_HANDSHAKE	0x0002	/* input hw handshake enabled */
#define FF_HDX_HANDSHAKE	0x0004	/* output hdx hw handshake enabled */
#define	FF_HWO_STOPPED		0x0008	/* output stopped by hw handshake */
#define FF_HWI_STARTED		0x0010	/* input started by hw handshake */
#define FF_HDX_STARTED		0x0020	/* output buffer contains characters */
#define FF_CARR_STOPPED		0x0040	/* output stopped by carrier detect */
#define FF_SWO_STOPPED		0x0080	/* output stopped by sw flow control */
#define FF_SWI_STOPPED		0x0100	/* input stopped by sw flow control */
#define FF_SW_FC_REQ		0x0200	/* sw input flow control request */
#define FF_RXFER_STOPPED	0x0400	/* rxfer function stopped */
#define FF_NEW_CTSRTS		0x0800	/* CTS/RTSFLOW do fdx hw handshake */
#define FF_DEF_HHO_LOW		0x1000	/* hw handshake outp. is low by deflt */
#define FF_OUTPUT_BUSY		0x2000	/* output buffers/circuits are busy */
#define FF_CD_ENABLED		0x4000	/* carrier detect enabled */
#define FF_CARRIER_ON		0x8000	/* carrier detect */

/* define the scheduled events flags */

#define EF_DO_RXFER		0x0001	/* rxfer function request */
#define EF_DO_XXFER		0x0002	/* xxfer function request */
#define EF_DO_BRKINT		0x0004	/* break int request */
#define EF_DO_MPROC		0x0008	/* mproc function request */
#define EF_RESET_DELTA_BITS	0x0010	/* reset accumulated msr delta bits */
#define EF_WAKEUP_VPIX		0x0020	/* wakeup VP/ix until rawq is empty */

/* define the device types
   The symbolic constant for the device type that allows the lowest interrupt
   latency (usually a 16450) has the lowest numerical value. Devices that
   allow a higher interrupt latency have a higher numerical value. With this
   ordering the device type can be used as a priority indicator during the
   interrupt processing, that is, devices that can't hold the received
   characters for very long are serviced first while devices with FIFOs
   are serviced only after all devices with higher priority are done.
   Usually, the following rule applies: The longer the FIFOs, the higher
   the assigned device type value should be, and the lower the interrupt
   priority will be.
*/

#define NUMBER_OF_TYPES		3	/* adjust this to the highest */
					/* device type + 1 */
#define TYPE_NS16450		0
#define TYPE_I82510		1
#define TYPE_NS16550A		2

/* modifier flags that can be set in fas_modify [] (`space.c') */

#define NO_TEST		0x0001		/* don't test the UART */
#define NO_HUP_PROTECT	0x0002		/* don't protect device after hangup */
#define NO_OVERRUN	0x0004		/* device is overrun protected */
#define NEW_CTSRTS	0x0008		/* CTS/RTSFLOW do fdx hw handshake */

/* read from port rather than write to it (for fas_init_seq [] and
   fas_int_ack_seq [] in `space.c')
*/

#define READ_PORT	0x0100

/* define the FIFO operating modes for the fas_fifo_ctl array in space.c
   (FIFO_POINTER_DEV and FIFO_TRIGGER_* apply to NS16550A UART, only)
*/

#define FIFO_DEFAULT		0	/* default mode (OS dependent) */
#define FIFO_OFF		1	/* 16450 compatible mode */
#define FIFO_EMUL_NS16450	2	/* emulate 16450 in FIFO mode */
#define FIFO_POINTER_DEV	3	/* pointer device (mouse etc.) */
#define FIFO_TRIGGER_1		4	/* set trigger level to 1 char */
#define FIFO_TRIGGER_4		5	/* set trigger level to 4 chars */
#define FIFO_TRIGGER_8		6	/* set trigger level to 8 chars */
#define FIFO_TRIGGER_14		7	/* set trigger level to 14 chars */

/* define an easy way to reference the port structures */

#define RCV_DATA_PORT		(fip->port [0])
#define XMT_DATA_PORT		(fip->port [0])
#define INT_ENABLE_PORT		(fip->port [1])
#define INT_ID_PORT		(fip->port [2])
#define NS_FIFO_CTL_PORT	(fip->port [2])
#define I_BANK_PORT		(fip->port [2])
#define LINE_CTL_PORT		(fip->port [3])
#define MDM_CTL_PORT		(fip->port [4])
#define I_IDM_PORT		(fip->port [4])
#define LINE_STATUS_PORT	(fip->port [5])
#define I_RCM_PORT		(fip->port [5])
#define MDM_STATUS_PORT		(fip->port [6])
#define I_TCM_PORT		(fip->port [6])
#define DIVISOR_LSB_PORT	(fip->port [0])
#define DIVISOR_MSB_PORT	(fip->port [1])
#define CTL_PORT		(fip->ctl_port)

/* UART related variables */

#define BT_SELECT		(fip->port [0].p.val)
#define IER			(fip->port [1].p.val)
#define FCR			(fip->port [2].p.val)
#define LCR			(fip->port [3].p.val)
#define MCR			(fip->port [4].p.val)
#define NEW_MSR			(fip->port [5].p.val)
#define MSR			(fip->port [6].p.val)
#define DEVICE_TYPE		(fip->ctl_port.p.val1)
#define INT_PRIO		(fip->ctl_port.p.val2)

/* ring buffer management */

#define RECV_SW_LOW_WATER	(fip->recv_sw_low_water)
#define RECV_SW_HIGH_WATER	(fip->recv_sw_high_water)
#define RECV_HW_LOW_WATER	(fip->recv_hw_low_water)
#define RECV_HW_HIGH_WATER	(fip->recv_hw_high_water)
#if defined (M_I286)
#define RECV_BUFF_SIZE		RECV_BUFF_MAX_SIZE
#define RECV_BUFF_BEGIN		(&fip->recv_buffer [0])
#define RECV_BUFF_END		(&fip->recv_buffer [RECV_BUFF_MAX_SIZE])
#define XMIT_BUFF_SIZE		XMIT_BUFF_MAX_SIZE
#define XMIT_BUFF_BEGIN		(&fip->xmit_buffer [0])
#define XMIT_BUFF_END		(&fip->xmit_buffer [XMIT_BUFF_MAX_SIZE])
#else
#define RECV_BUFF_SIZE		(fip->recv_buff_size)
#define RECV_BUFF_BEGIN		(fip->recv_buff_begin)
#define RECV_BUFF_END		(fip->recv_buff_end)
#define XMIT_BUFF_SIZE		(fip->xmit_buff_size)
#define XMIT_BUFF_BEGIN		(fip->xmit_buff_begin)
#define XMIT_BUFF_END		(fip->xmit_buff_end)
#endif

/* modem control port */

#define MC_SET_DTR		0x01
#define MC_SET_RTS		0x02
#define MC_SET_OUT1		0x04
#define MC_SET_OUT2		0x08
#define MC_SET_LOOPBACK		0x10

#define MC_ANY_CONTROL	(MC_SET_DTR | MC_SET_RTS | MC_SET_OUT1 | MC_SET_OUT2)

/* modem status port */

#define MS_CTS_DELTA		0x01
#define MS_DSR_DELTA		0x02
#define MS_RING_TEDGE		0x04
#define MS_DCD_DELTA		0x08
#define MS_CTS_PRESENT		0x10
#define MS_DSR_PRESENT		0x20
#define MS_RING_PRESENT		0x40
#define MS_DCD_PRESENT		0x80

#define MS_ANY_DELTA	(MS_CTS_DELTA | MS_DSR_DELTA | MS_RING_TEDGE \
				| MS_DCD_DELTA)
#define MS_ANY_PRESENT	(MS_CTS_PRESENT | MS_DSR_PRESENT | MS_RING_PRESENT \
				| MS_DCD_PRESENT)

/* interrupt enable port */

#define IE_NONE				0x00
#define	IE_RECV_DATA_AVAILABLE		0x01
#define	IE_XMIT_HOLDING_BUFFER_EMPTY	0x02
#define IE_LINE_STATUS			0x04
#define IE_MODEM_STATUS			0x08

/* interrupt id port */

#define II_NO_INTS_PENDING	0x01
#define II_CODE_MASK		0x07
#define II_MODEM_STATE		0x00
#define II_XMTD_CHAR		0x02
#define II_RCVD_CHAR		0x04
#define II_RCV_ERROR		0x06
#define II_NS_FIFO_TIMEOUT	0x08
#define II_NS_FIFO_ENABLED	0xC0

/* line control port */

#define	LC_WORDLEN_MASK		0x03
#define	LC_WORDLEN_5		0x00
#define	LC_WORDLEN_6		0x01
#define	LC_WORDLEN_7		0x02
#define	LC_WORDLEN_8		0x03
#define LC_STOPBITS_LONG	0x04
#define LC_ENABLE_PARITY	0x08
#define LC_EVEN_PARITY		0x10
#define LC_STICK_PARITY		0x20
#define LC_SET_BREAK_LEVEL	0x40
#define LC_ENABLE_DIVISOR	0x80

/* line status port */

#define LS_RCV_AVAIL		0x01
#define LS_OVERRUN		0x02
#define LS_PARITY_ERROR		0x04
#define LS_FRAMING_ERROR	0x08
#define LS_BREAK_DETECTED	0x10
#define LS_XMIT_AVAIL		0x20
#define LS_XMIT_COMPLETE	0x40
#define LS_ERROR_IN_NS_FIFO	0x80	/* NS16550A only */
 
#define LS_RCV_INT	(LS_RCV_AVAIL | LS_OVERRUN | LS_PARITY_ERROR \
			| LS_FRAMING_ERROR | LS_BREAK_DETECTED)

/* fifo control port (NS16550A only) */

#define	NS_FIFO_ENABLE		0x01
#define	NS_FIFO_CLR_RECV	0x02
#define	NS_FIFO_CLR_XMIT	0x04
#define	NS_FIFO_START_DMA	0x08
#define NS_FIFO_SIZE_1		0x00
#define NS_FIFO_SIZE_4		0x40
#define NS_FIFO_SIZE_8		0x80
#define NS_FIFO_SIZE_14		0xC0
#define NS_FIFO_SIZE_MASK	0xC0

#define NS_FIFO_CLEAR_CMD	0
#define NS_FIFO_INIT_CMD	(NS_FIFO_SIZE_1 | NS_FIFO_ENABLE \
				| NS_FIFO_CLR_RECV | NS_FIFO_CLR_XMIT)

#define INPUT_NS_FIFO_SIZE	16
#define OUTPUT_NS_FIFO_SIZE	16

/* fifo control ports (i82510 only) */

#define I_BANK_0		0x00
#define I_BANK_1		0x20
#define I_BANK_2		0x40
#define I_BANK_3		0x60
#define I_FIFO_ENABLE		0x08
#define I_FIFO_CLR_RECV		0x30
#define I_FIFO_CLR_XMIT		0x0c

#define I_FIFO_CLEAR_CMD	0
#define I_FIFO_SETUP_CMD	I_FIFO_ENABLE

#define INPUT_I_FIFO_SIZE	4
#define OUTPUT_I_FIFO_SIZE	4

/* defines for ioctl calls (VP/ix) */

#define AIOC			('A'<<8)
#define AIOCINTTYPE		(AIOC|60)	/* set interrupt type */
#define AIOCDOSMODE		(AIOC|61)	/* set DOS mode */
#define AIOCNONDOSMODE		(AIOC|62)	/* reset DOS mode */
#define AIOCSERIALOUT		(AIOC|63)	/* serial device data write */
#define AIOCSERIALIN		(AIOC|64)	/* serial device data read */
#define AIOCSETSS		(AIOC|65)	/* set start/stop chars */
#define AIOCINFO		(AIOC|66)	/* tell us what device we are */

/* ioctl alternate names used by VP/ix */

#define VPC_SERIAL_DOS		AIOCDOSMODE
#define VPC_SERIAL_NONDOS	AIOCNONDOSMODE
#define VPC_SERIAL_INFO		AIOCINFO
#define VPC_SERIAL_OUT		AIOCSERIALOUT
#define VPC_SERIAL_IN		AIOCSERIALIN

/* serial in/out requests */

#define SO_DIVLLSB		1
#define SO_DIVLMSB		2
#define SO_LCR			3
#define SO_MCR			4
#define SI_MSR			1
#define SIO_MASK(x)		(1<<((x)-1))


/* This structure contains baud rate dependent informations. */

struct fas_speed
{
	union {
		uint	ctime;	/* kernel clock ticks until xmitter is empty */
		uint	max_rate;	/* max baud rate index */
	} i;
	union {			/* UART counter divisor value */
		struct {
			unchar	low;	/* low byte */
			unchar	high;	/* high byte */
		} b;
		ushort	val;
	} div;
	ushort	xbuf_size;	/* xmit buffer size that fits the baud rate */
};


/* This structure contains everything one would like to know about
   an open device.  There is one of it for each physical unit.

   Take care that the size of the area that contains the various
   structure fields (up to, but excluding the ring buffers)
   is <= 128 bytes. Otherwise a 4-byte offset is used to access
   some of the structure fields. For the first 128 bytes a 1-byte
   offset is used, which is faster.
*/

struct fas_internals
{
	struct tty	*tty;	/* the tty structure */
	struct fas_internals	*next_int_user;	/* link to next struct */
	struct fas_internals	*prev_int_user;	/* link to previous struct */
	uint	device_flags;	/* flags about the device state */
	uint	flow_flags;	/* flags about the flow control state */
	uint	event_flags;	/* flags about scheduled events */
	uint	o_state;	/* current open state */
	uint	po_state;	/* previous open state */
	uint	iflag;		/* current terminal input flags */
	uint	cflag;		/* current terminal hardware control flags */
	union {			/* modem control masks */
		struct {
			unchar	di;	/* mask for modem disable */
			unchar	eo;	/* mask for modem enable (dialout) */
			unchar	ei;	/* mask for modem enable (dialin) */
			unchar	ca;	/* mask for carrier detect */
		} m;
		ulong	l;
	} modem;
	union {			/* hardware flow control masks */
		struct {
			unchar	ic;	/* control mask for inp. flow ctrl */
			unchar	oc;	/* control mask for outp. flow ctrl */
			unchar	oe;	/* enable mask for outp. flow ctrl */
			unchar	hc;	/* control mask for hdx flow ctrl */
		} m;
		ulong	l;
	} flow;
	union {			/* muliplexer control port */
		uint	addr;
		struct {
			ushort	addr;
			unchar	val1;
			unchar	val2;
		} p;
	} ctl_port;
	union {			/* uart port addresses and control values */
		uint	addr;
		struct {
			ushort	addr;
			unchar	ctl;
			unchar	val;
		} p;
	} port [NUM_UART_REGS];
	int	timeout_idx;	/* timeout index for untimeout () */
	uint	rxfer_timeout;	/* timeout counter for recv char transfer */
	unchar	start_char;	/* start character for software flow control */
	unchar	stop_char;	/* stop character for software flow control */
#if defined (HAVE_VPIX)
	unchar	v86_intmask;	/* VP/ix pseudorupt mask */
	v86_t	*v86_proc;	/* VP/ix v86proc pointer for pseudorupts */
#endif
	uint	recv_ring_cnt;	/* receiver ring buffer counter */
	unchar	*recv_ring_put_ptr;	/* recv ring buf put ptr */
	unchar	*recv_ring_take_ptr;	/* recv ring buf take ptr */
	uint	xmit_fifo_size;	/* transmitter FIFO size */
	uint	xmit_ring_max;	/* transmitter ring buffer max fill level */
	uint	xmit_ring_cnt;	/* transmitter ring buffer counter */
	unchar	*xmit_ring_put_ptr;	/* xmit ring buf put ptr */
	unchar	*xmit_ring_take_ptr;	/* xmit ring buf take ptr */
	uint	recv_sw_low_water;	/* recv software fc low water level */
	uint	recv_sw_high_water;	/* recv software fc high water level */
	uint	recv_hw_low_water;	/* recv hardware fc low water level */
	uint	recv_hw_high_water;	/* recv hardware fc high water level */
#if defined (M_I286)
	unchar	recv_buffer [RECV_BUFF_MAX_SIZE];	/* recv ring buf */
	unchar	xmit_buffer [XMIT_BUFF_MAX_SIZE];	/* xmit ring buf */
#else
	uint	recv_buff_size;		/* recv ring buffer size */
	unchar	*recv_buff_begin;	/* recv ring buffer begin ptr */
	unchar	*recv_buff_end;		/* recv ring buffer end ptr */
	uint	xmit_buff_size;		/* xmit ring buffer size */
	unchar	*xmit_buff_begin;	/* xmit ring buffer begin ptr */
	unchar	*xmit_buff_end;		/* xmit ring buffer end ptr */
#endif
};
