# optim_att.awk
#
# filter AT&T assembler file format
#
# Replace all calls to outb(), inb() and fas_mem_copy() with the necessary
# assembler mnemonics and do some optimizations for bit tests.

BEGIN {
	nl = 0
	prevline = ""
}

{
	if ($1 == "popl") {
	    print prevline
	    if ($2 == "%ecx" && split(prevline, argsp) > 0 \
			&& argsp[1] == "call") {
		prevline = "\taddl\t$4,%esp"
	    } else {
		prevline = $0
	    }
	    next
	} else if ($1 == "pushl") {
	    if ($2 == "%eax" && split(prevline, argsp) == 2 \
			&& argsp[1] == "movl" && argsp[2] == "%esp,%ebp") {
		print prevline
		prevline = "\tsubl\t$4,%esp"
		next
	    }
	    lines[nl++] = $0
	    getline
	    if ($1 == "pushl") {
		lines[nl++] = $0
		getline
		if ($1 == "call" && $2 == "outb") {
		    lines[nl++] = $0
		    getline
		    if ($1 == "addl" && $2 == "$8,%esp") {
			split(lines[0], args0)
			split(lines[1], args1)
			if (args1[2] == "%eax") {
			    print prevline
			    if (args0[2] == "%edx") {
				print "\tmovl\t%eax,%ecx"
				print "\tmovl\t%edx,%eax"
				print "\tmovl\t%ecx,%edx"
			    } else {
				print "\tmovl\t%eax,%edx"
				print "\tmovl\t" args0[2] ",%eax"
			    }
			} else {
			    if (args0[2] == "%eax") {
				res = 0
				if (prevline ~ /,%eax$/) {
				    argsline = substr(prevline, 1, \
							length(prevline) - 5) \
						" %eax"
				    split(argsline, argsp)
				    res = 1
				}
				if (res && argsp[3] == "%eax") {
				    if (argsp[1] == "movzbl" \
						|| argsp[1] == "movsbl") {
					if (argsp[2] != "%al") {
					    print "\tmovb\t" argsp[2] ",%al"
					}
				    } else if (argsp[1] == "movzwl" \
						|| argsp[1] == "movswl") {
					if (argsp[2] != "%ax") {
					    if (argsp[2] ~ /^%/) {
						print "\tmovw\t" argsp[2] ",%ax"
					    } else {
						print "\tmovb\t" argsp[2] ",%al"
					    }
					}
				    } else {
					print prevline
				    }
				} else {
				    print prevline
				}
			    } else {
				print prevline
				if (args0[2] ~ /^%/) {
				    print "\tmovl\t" args0[2] ",%eax"
				} else {
				    print "\tmovb\t" args0[2] ",%al"
				}
			    }
			    if (args1[2] != "%edx") {
				print "\tmovl\t" args1[2] ",%edx"
			    }
			}
			prevline = "\toutb\t(%dx)"
			nl = 0
			next
		    }
		} else if ($1 == "pushl") {
		    lines[nl++] = $0
		    getline
		    if ($1 == "call" && $2 == "fas_mem_copy") {
			lines[nl++] = $0
			getline
			if ($1 == "addl" && $2 == "$12,%esp") {
			    split(lines[0], args0)
			    split(lines[1], args1)
			    split(lines[2], args2)
			    print prevline
			    print "\tmovl\t" args2[2] ",%edi"
			    print "\tmovl\t" args1[2] ",%esi"
			    reg = ""
			    if (args0[2] != "%ecx") {
				if (args0[2] ~ /^[%$]/) {
				    reg = args0[2]
				}
				print "\tmovl\t" args0[2] ",%ecx"
			    }
			    print "\tcld"
			    if (reg == "") {
				reg = "%eax"
				print "\tmovl\t%ecx,%eax"
			    }
			    print "\tshrl\t$2,%ecx"
			    print "\trep"
			    print "\tmovsl"
			    print "\tmovl\t" reg ",%ecx"
			    print "\tandl\t$3,%ecx"
			    print "\trep"
			    prevline = "\tmovsb"
			    nl = 0
			    next
			}
		    }
		}
	    } else if ($1 == "call" && $2 == "inb") {
		lines[nl++] = $0
		getline
		if ($1 == "popl" || ($1 == "addl" && $2 == "$4,%esp")) {
		    split(lines[0], args0)
		    print prevline
		    if (args0[2] != "%edx") {
			print "\tmovl\t" args0[2] ",%edx"
		    }
		    print "\txorl\t%eax,%eax"
		    prevline = "\tinb\t(%dx)"
		    nl = 0
		    next
		}
	    }

	    print prevline
	    prevline = $0

	    for (i = 0; i < nl; i++) {
		print lines[i]
	    }
	    nl = 0
	} else {
	    print prevline
	    prevline = $0
	}

	cmd = substr($1, 1, length($1) - 1)
	class = substr($1, length($1))

	if (cmd == "test" && (class == "l" || class == "w") && $2 ~ /^\$/) {
	    split($2, args, ",")
	    args[2] = substr($2, length(args[1]) + 2)
	    if (args[2] ~ /^%.*x$/) {
		reg = substr(args[2], length(args[2]) - 1, 1)
	    } else {
		reg = ""
	    }
	    if (reg != "" || args[2] !~ /^%/) {
		val = substr(args[1], 2) + 0
		if (class == "w" && val <= -1 && val >= -65536) {
		    val += 65536
		}
		if (val >= 0 && val <= 255) {
		    if (reg != "") {
			prevline = "\t" cmd "b\t" args[1] ",%" reg "l"
		    } else {
			prevline = "\t" cmd "b\t" args[1] "," args[2]
		    }
		} else if (val >= 0 && val <= 65535 && (val % 256) == 0) {
		    if (reg != "") {
			prevline = "\t" cmd "b\t$" val / 256 ",%" reg "h"
		    } else if (args[2] ~ /^[-(]/) {
			prevline = "\t" cmd "b\t$" val / 256 ",1" args[2]
		    } else {
			prevline = "\t" cmd "b\t$" val / 256 ",1+" args[2]
		    }
		}
	    }
	}
}

END {
	    print prevline
}
